@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import ast
 import re
 import time
 
@@ -33,8 +34,10 @@ MESSAGE = 'message'
 SHARE = 'share'
 SHARE_TYPE = 'share_type'
 SHARE_NETWORK = 'share_network'
+SHARE_NETWORK_SUBNET = 'share_network_subnet'
 SHARE_SERVER = 'share_server'
 SNAPSHOT = 'snapshot'
+SHARE_REPLICA = 'share_replica'
 
 
 def not_found_wrapper(f):
@@ -43,8 +46,9 @@ def not_found_wrapper(f):
         try:
             return f(self, *args, **kwargs)
         except tempest_lib_exc.CommandFailed as e:
-            for regexp in ('No (\w+) with a name or ID', 'not(.*){0,5}found'):
-                if re.search(regexp, e.stderr):
+            for regexp in (r'No (\w+) with a name or ID',
+                           r'not(.*){0,5}found'):
+                if re.search(regexp, six.text_type(e.stderr)):
                     # Raise appropriate 'NotFound' error
                     raise tempest_lib_exc.NotFound()
             raise
@@ -58,7 +62,7 @@ def forbidden_wrapper(f):
         try:
             return f(self, *args, **kwargs)
         except tempest_lib_exc.CommandFailed as e:
-            if re.search('HTTP 403', e.stderr):
+            if re.search('HTTP 403', six.text_type(e.stderr)):
                 # Raise appropriate 'Forbidden' error.
                 raise tempest_lib_exc.Forbidden()
             raise
@@ -114,20 +118,21 @@ class ManilaCLIClient(base.CLIClient):
             'manila', action, flags, params, fail_ok, merge_stderr)
 
     def wait_for_resource_deletion(self, res_type, res_id, interval=3,
-                                   timeout=180, microversion=None):
+                                   timeout=180, microversion=None, **kwargs):
         """Resource deletion waiter.
 
-        :param res_type: text -- type of resource. Supported only 'share_type'.
-            Other types support is TODO.
+        :param res_type: text -- type of resource
         :param res_id: text -- ID of resource to use for deletion check
         :param interval: int -- interval between requests in seconds
         :param timeout: int -- total time in seconds to wait for deletion
+        :param args: dict -- additional keyword arguments for deletion func
         """
-        # TODO(vponomaryov): add support for other resource types
         if res_type == SHARE_TYPE:
             func = self.is_share_type_deleted
         elif res_type == SHARE_NETWORK:
             func = self.is_share_network_deleted
+        elif res_type == SHARE_NETWORK_SUBNET:
+            func = self.is_share_network_subnet_deleted
         elif res_type == SHARE_SERVER:
             func = self.is_share_server_deleted
         elif res_type == SHARE:
@@ -136,15 +141,17 @@ class ManilaCLIClient(base.CLIClient):
             func = self.is_snapshot_deleted
         elif res_type == MESSAGE:
             func = self.is_message_deleted
+        elif res_type == SHARE_REPLICA:
+            func = self.is_share_replica_deleted
         else:
             raise exceptions.InvalidResource(message=res_type)
 
         end_loop_time = time.time() + timeout
-        deleted = func(res_id, microversion=microversion)
+        deleted = func(res_id, microversion=microversion, **kwargs)
 
         while not (deleted or time.time() > end_loop_time):
             time.sleep(interval)
-            deleted = func(res_id, microversion=microversion)
+            deleted = func(res_id, microversion=microversion, **kwargs)
 
         if not deleted:
             raise exceptions.ResourceReleaseFailed(
@@ -241,6 +248,41 @@ class ManilaCLIClient(base.CLIClient):
         share_type = utils.details(share_type_raw)
         return share_type
 
+    def update_share_type(self, share_type_id, name=None,
+                          is_public=None, microversion=None,
+                          description=None):
+        """Update share type.
+
+        :param share_type_id: text -- id of share type.
+        :param name: text -- new name of share type, if not set then
+            it will not be updated.
+        :param description: text -- new description of share type.
+            if not set then it will not be updated.
+        :param is_public: bool/str -- boolean or its string alias.
+            new visibility of the share type.If set to True, share
+            type will be available to all tenants in the cloud.
+        """
+
+        cmd = ('type-update %(share_type_id)s ') % {
+            'share_type_id': share_type_id}
+
+        if is_public is not None:
+            if not isinstance(is_public, six.string_types):
+                is_public = six.text_type(is_public)
+            cmd += " --is_public " + is_public
+
+        if description:
+            cmd += " --description " + description
+        elif description == "":
+            cmd += ' --description "" '
+
+        if name:
+            cmd += " --name " + name
+
+        share_type_raw = self.manila(cmd, microversion=microversion)
+        share_type = utils.details(share_type_raw)
+        return share_type
+
     @not_found_wrapper
     def delete_share_type(self, share_type, microversion=None):
         """Deletes share type by its Name or ID."""
@@ -274,12 +316,15 @@ class ManilaCLIClient(base.CLIClient):
     def get_share_type(self, share_type, microversion=None):
         """Get share type.
 
-        :param share_type: str -- Name or ID of share type
+        :param share_type: str -- Name or ID of share type, or None to
+                    retrieve default share type
         """
         share_types = self.list_share_types(True, microversion=microversion)
-        for st in share_types:
-            if share_type in (st['ID'], st['Name']):
-                return st
+        for stype in share_types:
+            if share_type is None and stype["is_default"] == 'YES':
+                return stype
+            elif share_type in (stype['ID'], stype['Name']):
+                return stype
         raise tempest_lib_exc.NotFound()
 
     def is_share_type_deleted(self, share_type, microversion=None):
@@ -410,7 +455,8 @@ class ManilaCLIClient(base.CLIClient):
 
     def create_share_network(self, name=None, description=None,
                              nova_net_id=None, neutron_net_id=None,
-                             neutron_subnet_id=None, microversion=None):
+                             neutron_subnet_id=None, availability_zone=None,
+                             microversion=None):
         """Creates share network.
 
         :param name: text -- desired name of new share network
@@ -427,7 +473,9 @@ class ManilaCLIClient(base.CLIClient):
             description=description,
             nova_net_id=nova_net_id,
             neutron_net_id=neutron_net_id,
-            neutron_subnet_id=neutron_subnet_id)
+            neutron_subnet_id=neutron_subnet_id,
+            availability_zone=availability_zone
+        )
         share_network_raw = self.manila(
             'share-network-create %s' % params, microversion=microversion)
         share_network = output_parser.details(share_network_raw)
@@ -435,7 +483,8 @@ class ManilaCLIClient(base.CLIClient):
 
     def _combine_share_network_data(self, name=None, description=None,
                                     nova_net_id=None, neutron_net_id=None,
-                                    neutron_subnet_id=None):
+                                    neutron_subnet_id=None,
+                                    availability_zone=None):
         """Combines params for share network operations 'create' and 'update'.
 
         :returns: text -- set of CLI parameters
@@ -451,9 +500,11 @@ class ManilaCLIClient(base.CLIClient):
             data['--neutron_net_id'] = neutron_net_id
         if neutron_subnet_id is not None:
             data['--neutron_subnet_id'] = neutron_subnet_id
+        if availability_zone is not None:
+            data['--availability_zone'] = availability_zone
         cmd = ''
         for key, value in data.items():
-            cmd += "%(k)s=%(v)s " % dict(k=key, v=value)
+            cmd += "%(k)s=%(v)s " % {'k': key, 'v': value}
         return cmd
 
     @not_found_wrapper
@@ -512,7 +563,7 @@ class ManilaCLIClient(base.CLIClient):
         """List share networks.
 
         :param all_tenants: bool -- whether to list share-networks that belong
-            only to current project or for all projects.
+            only to current project or for all tenants.
         :param filters: dict -- filters for listing of share networks.
             Example, input:
                 {'project_id': 'foo'}
@@ -557,6 +608,106 @@ class ManilaCLIClient(base.CLIClient):
             SHARE_NETWORK, res_id=share_network, interval=2, timeout=6,
             microversion=microversion)
 
+    # Share Network Subnets
+
+    def _combine_share_network_subnet_data(self, neutron_net_id=None,
+                                           neutron_subnet_id=None,
+                                           availability_zone=None):
+        """Combines params for share network subnet 'create' operation.
+
+        :returns: text -- set of CLI parameters
+        """
+        data = dict()
+        if neutron_net_id is not None:
+            data['--neutron_net_id'] = neutron_net_id
+        if neutron_subnet_id is not None:
+            data['--neutron_subnet_id'] = neutron_subnet_id
+        if availability_zone is not None:
+            data['--availability_zone'] = availability_zone
+        cmd = ''
+        for key, value in data.items():
+            cmd += "%(k)s=%(v)s " % dict(k=key, v=value)
+        return cmd
+
+    def add_share_network_subnet(self, share_network,
+                                 neutron_net_id=None, neutron_subnet_id=None,
+                                 availability_zone=None, microversion=None):
+        """Create new share network subnet for the given share network."""
+        params = self._combine_share_network_subnet_data(
+            neutron_net_id=neutron_net_id,
+            neutron_subnet_id=neutron_subnet_id,
+            availability_zone=availability_zone)
+        share_network_subnet_raw = self.manila(
+            'share-network-subnet-create %(sn)s %(params)s' %
+            {'sn': share_network, 'params': params}, microversion=microversion)
+        share_network_subnet = output_parser.details(share_network_subnet_raw)
+        return share_network_subnet
+
+    def get_share_network_subnet(self, share_network, share_network_subnet,
+                                 microversion=None):
+        """Returns share network subnet by share network ID and subnet ID."""
+
+        share_network_subnet_raw = self.manila(
+            'share-network-subnet-show %(share_net)s %(share_subnet)s' % {
+                'share_net': share_network,
+                'share_subnet': share_network_subnet,
+            })
+        share_network_subnet = output_parser.details(share_network_subnet_raw)
+        return share_network_subnet
+
+    def get_share_network_subnets(self, share_network, microversion=None):
+        share_network = self.get_share_network(share_network,
+                                               microversion=microversion)
+        raw_subnets = share_network.get('share_network_subnets')
+        subnets = ast.literal_eval(raw_subnets)
+        # NOTE(lseki): convert literal None to string 'None'
+        for subnet in subnets:
+            for k, v in subnet.items():
+                subnet[k] = str(v) if v is None else v
+        return subnets
+
+    @not_found_wrapper
+    def delete_share_network_subnet(self, share_network, share_network_subnet,
+                                    microversion=None):
+        """Delete a share_network."""
+        self.manila(
+            'share-network-subnet-delete %(share_net)s %(share_subnet)s' % {
+                'share_net': share_network,
+                'share_subnet': share_network_subnet,
+            }, microversion=microversion)
+
+    def is_share_network_subnet_deleted(self, share_network_subnet,
+                                        share_network, microversion=None):
+        # NOTE(lseki): the parameter share_network_subnet comes before
+        # share_network, because the wrapper method wait_for_resource_deletion
+        # expects the resource id in first place (subnet ID in this case).
+        """Says whether share network subnet is deleted or not.
+
+        :param share_network_subnet: text -- Name or ID of share network subnet
+        :param share_network: text -- Name or ID of share network the subnet
+            belongs to
+        """
+        subnets = self.get_share_network_subnets(share_network)
+        return not any(subnet['id'] == share_network_subnet
+                       for subnet in subnets)
+
+    def wait_for_share_network_subnet_deletion(self, share_network_subnet,
+                                               share_network,
+                                               microversion=None):
+        # NOTE(lseki): the parameter share_network_subnet comes before
+        # share_network, because the wrapper method wait_for_resource_deletion
+        # expects the resource id in first place (subnet ID in this case).
+        """Wait for share network subnet deletion by its Name or ID.
+
+        :param share_network_subnet: text -- Name or ID of share network subnet
+        :param share_network: text -- Name or ID of share network the subnet
+            belongs to
+        """
+        args = {'share_network': share_network}
+        self.wait_for_resource_deletion(
+            SHARE_NETWORK_SUBNET, res_id=share_network_subnet,
+            interval=2, timeout=6, microversion=microversion, **args)
+
     # Shares
 
     def create_share(self, share_protocol, size, share_network=None,
@@ -651,7 +802,7 @@ class ManilaCLIClient(base.CLIClient):
         """List shares.
 
         :param all_tenants: bool -- whether to list shares that belong
-            only to current project or for all projects.
+            only to current project or for all tenants.
         :param filters: dict -- filters for listing of shares.
             Example, input:
                 {'project_id': 'foo'}
@@ -724,29 +875,30 @@ class ManilaCLIClient(base.CLIClient):
             SHARE, res_id=share, interval=5, timeout=300,
             microversion=microversion)
 
-    def wait_for_share_status(self, share, status, microversion=None):
+    def wait_for_resource_status(self, resource_id, status, microversion=None,
+                                 resource_type="share"):
         """Waits for a share to reach a given status."""
-        body = self.get_share(share, microversion=microversion)
-        share_name = body['name']
+        get_func = getattr(self, 'get_' + resource_type)
+        body = get_func(resource_id, microversion=microversion)
         share_status = body['status']
         start = int(time.time())
 
         while share_status != status:
             time.sleep(self.build_interval)
-            body = self.get_share(share, microversion=microversion)
+            body = get_func(resource_id, microversion=microversion)
             share_status = body['status']
 
             if share_status == status:
                 return
             elif 'error' in share_status.lower():
-                raise exceptions.ShareBuildErrorException(share=share)
+                raise exceptions.ShareBuildErrorException(share=resource_id)
 
             if int(time.time()) - start >= self.build_timeout:
-                message = (
-                    "Share %(share_name)s failed to reach %(status)s status "
-                    "within the required time (%(build_timeout)s s)." % {
-                        "share_name": share_name, "status": status,
-                        "build_timeout": self.build_timeout})
+                message = ("Resource %(resource_id)s failed to reach "
+                           "%(status)s  status within the required time "
+                           "(%(build_timeout)s)." %
+                           {"resource_id": resource_id, "status": status,
+                            "build_timeout": self.build_timeout})
                 raise tempest_lib_exc.TimeoutException(message)
 
     def wait_for_migration_task_state(self, share_id, dest_host,
@@ -1196,7 +1348,7 @@ class ManilaCLIClient(base.CLIClient):
         return output_parser.listing(response)
 
     def create_security_service(self, type='ldap', name=None, description=None,
-                                dns_ip=None, server=None, domain=None,
+                                dns_ip=None, ou=None, server=None, domain=None,
                                 user=None, password=None, microversion=None):
         """Creates security service.
 
@@ -1204,6 +1356,7 @@ class ManilaCLIClient(base.CLIClient):
         :param name: desired name of new security service.
         :param description: desired description of new security service.
         :param dns_ip: DNS IP address inside tenant's network.
+        :param ou: security service organizational unit
         :param server: security service IP address or hostname.
         :param domain: security service domain.
         :param user: user of the new security service.
@@ -1215,6 +1368,7 @@ class ManilaCLIClient(base.CLIClient):
             name=name,
             description=description,
             dns_ip=dns_ip,
+            ou=ou,
             server=server,
             domain=domain,
             user=user,
@@ -1226,14 +1380,15 @@ class ManilaCLIClient(base.CLIClient):
 
     @not_found_wrapper
     def update_security_service(self, security_service, name=None,
-                                description=None, dns_ip=None, server=None,
-                                domain=None, user=None, password=None,
-                                microversion=None):
+                                description=None, dns_ip=None, ou=None,
+                                server=None, domain=None, user=None,
+                                password=None, microversion=None):
         cmd = 'security-service-update %s ' % security_service
         cmd += self. _combine_security_service_data(
             name=name,
             description=description,
             dns_ip=dns_ip,
+            ou=ou,
             server=server,
             domain=domain,
             user=user,
@@ -1242,8 +1397,8 @@ class ManilaCLIClient(base.CLIClient):
             self.manila(cmd, microversion=microversion))
 
     def _combine_security_service_data(self, name=None, description=None,
-                                       dns_ip=None, server=None, domain=None,
-                                       user=None, password=None):
+                                       dns_ip=None, ou=None, server=None,
+                                       domain=None, user=None, password=None):
         data = ''
         if name is not None:
             data += '--name %s ' % name
@@ -1251,6 +1406,8 @@ class ManilaCLIClient(base.CLIClient):
             data += '--description %s ' % description
         if dns_ip is not None:
             data += '--dns-ip %s ' % dns_ip
+        if ou is not None:
+            data += '--ou %s ' % ou
         if server is not None:
             data += '--server %s ' % server
         if domain is not None:
@@ -1430,6 +1587,109 @@ class ManilaCLIClient(base.CLIClient):
             SHARE_SERVER, res_id=share_server, interval=3, timeout=60,
             microversion=microversion)
 
+    def unmanage_share(self, server_id):
+        return self.manila('unmanage %s ' % server_id)
+
+    def unmanage_server(self, share_server_id):
+        return self.manila('share-server-unmanage %s ' % share_server_id)
+
+    def share_server_manage(self, host, share_network, identifier,
+                            driver_options=None):
+        if driver_options:
+            command = ('share-server-manage %s %s %s %s' %
+                       (host, share_network, identifier, driver_options))
+        else:
+            command = ('share-server-manage %s %s %s' % (host, share_network,
+                       identifier))
+        managed_share_server_raw = self.manila(command)
+        managed_share_server = output_parser.details(managed_share_server_raw)
+        return managed_share_server['id']
+
+    def manage_share(self, host, protocol, export_location, share_server):
+        managed_share_raw = self.manila(
+            'manage %s %s %s --share-server-id %s' % (host, protocol,
+                                                      export_location,
+                                                      share_server))
+        managed_share = output_parser.details(managed_share_raw)
+        return managed_share['id']
+
+    def share_server_migration_check(self, server_id, dest_host, writable,
+                                     nondisruptive, preserve_snapshots,
+                                     new_share_network=None):
+        cmd = ('share-server-migration-check %(server_id)s %(host)s '
+               '--writable %(writable)s --nondisruptive %(nondisruptive)s '
+               '--preserve-snapshots %(preserve_snapshots)s') % {
+            'server_id': server_id,
+            'host': dest_host,
+            'writable': writable,
+            'nondisruptive': nondisruptive,
+            'preserve_snapshots': preserve_snapshots,
+        }
+        if new_share_network:
+            cmd += ' --new-share-network %s' % new_share_network
+        result = self.manila(cmd)
+        return output_parser.details(result)
+
+    def share_server_migration_start(self, server_id, dest_host,
+                                     writable=False, nondisruptive=False,
+                                     preserve_snapshots=False,
+                                     new_share_network=None):
+        cmd = ('share-server-migration-start %(server_id)s %(host)s '
+               '--writable %(writable)s --nondisruptive %(nondisruptive)s '
+               '--preserve-snapshots %(preserve_snapshots)s') % {
+            'server_id': server_id,
+            'host': dest_host,
+            'writable': writable,
+            'nondisruptive': nondisruptive,
+            'preserve_snapshots': preserve_snapshots,
+        }
+        if new_share_network:
+            cmd += ' --new-share-network %s' % new_share_network
+        return self.manila(cmd)
+
+    def share_server_migration_complete(self, server_id):
+        return self.manila('share-server-migration-complete %s' % server_id)
+
+    def share_server_migration_cancel(self, server_id):
+        return self.manila('share-server-migration-cancel %s' % server_id)
+
+    def share_server_migration_get_progress(self, server_id):
+        result = self.manila('share-server-migration-get-progress %s'
+                             % server_id)
+        return output_parser.details(result)
+
+    def wait_for_server_migration_task_state(self, share_server_id, dest_host,
+                                             task_state_to_wait,
+                                             microversion=None):
+        """Waits for a certain server task state. """
+        statuses = ((task_state_to_wait,)
+                    if not isinstance(task_state_to_wait, (tuple, list, set))
+                    else task_state_to_wait)
+        server = self.get_share_server(share_server=share_server_id,
+                                       microversion=microversion)
+        start = int(time.time())
+        while server['task_state'] not in statuses:
+            time.sleep(self.build_interval)
+            server = self.get_share_server(share_server=share_server_id,
+                                           microversion=microversion)
+            if server['task_state'] in statuses:
+                return server
+            elif server['task_state'] == constants.TASK_STATE_MIGRATION_ERROR:
+                raise exceptions.ShareServerMigrationException(
+                    server_id=server['id'])
+            elif int(time.time()) - start >= self.build_timeout:
+                message = ('Server %(share_server_id)s failed to reach the '
+                           'status in %(status)s while migrating from host '
+                           '%(src)s to host %(dest)s within the required time '
+                           '%(timeout)s.' % {
+                               'src': server['host'],
+                               'dest': dest_host,
+                               'share_server_id': server['id'],
+                               'timeout': self.build_timeout,
+                               'status': six.text_type(statuses),
+                           })
+                raise tempest_lib_exc.TimeoutException(message)
+
     # user messages
 
     def wait_for_message(self, resource_id):
@@ -1496,3 +1756,118 @@ class ManilaCLIClient(base.CLIClient):
         self.wait_for_resource_deletion(
             MESSAGE, res_id=message, interval=3, timeout=60,
             microversion=microversion)
+
+    # Share replicas
+
+    def create_share_replica(self, share, microversion=None):
+        """Create a share replica.
+
+        :param share: str -- Name or ID of a share to create a replica of
+        """
+        cmd = "share-replica-create %s" % share
+        replica = self.manila(cmd, microversion=microversion)
+        return output_parser.details(replica)
+
+    @not_found_wrapper
+    def get_share_replica(self, replica, microversion=None):
+        cmd = "share-replica-show %s" % replica
+        replica = self.manila(cmd, microversion=microversion)
+        return output_parser.details(replica)
+
+    @not_found_wrapper
+    @forbidden_wrapper
+    def delete_share_replica(self, share_replica, microversion=None):
+        """Deletes share replica by ID."""
+        return self.manila(
+            "share-replica-delete %s" % share_replica,
+            microversion=microversion)
+
+    def is_share_replica_deleted(self, replica, microversion=None):
+        """Indicates whether a share replica is deleted or not.
+
+        :param replica: str -- ID of share replica
+        """
+        try:
+            self.get_share_replica(replica, microversion=microversion)
+            return False
+        except tempest_lib_exc.NotFound:
+            return True
+
+    def wait_for_share_replica_deletion(self, replica, microversion=None):
+        """Wait for share replica deletion by its ID.
+
+        :param replica: text -- ID of share replica
+        """
+        self.wait_for_resource_deletion(
+            SHARE_REPLICA, res_id=replica, interval=3, timeout=60,
+            microversion=microversion)
+
+    def wait_for_share_replica_status(self, share_replica,
+                                      status="available",
+                                      microversion=None):
+        """Waits for a share replica to reach a given status."""
+        replica = self.get_share_replica(share_replica,
+                                         microversion=microversion)
+        share_replica_status = replica['status']
+        start = int(time.time())
+
+        while share_replica_status != status:
+            time.sleep(self.build_interval)
+            replica = self.get_share_replica(share_replica,
+                                             microversion=microversion)
+            share_replica_status = replica['status']
+
+            if share_replica_status == status:
+                return replica
+            elif 'error' in share_replica_status.lower():
+                raise exceptions.ShareReplicaBuildErrorException(
+                    replica=share_replica)
+
+            if int(time.time()) - start >= self.build_timeout:
+                message = (
+                    "Share replica %(id)s failed to reach %(status)s "
+                    "status within the required time "
+                    "(%(build_timeout)s s)." % {
+                        "id": share_replica, "status": status,
+                        "build_timeout": self.build_timeout})
+                raise tempest_lib_exc.TimeoutException(message)
+        return replica
+
+    @not_found_wrapper
+    @forbidden_wrapper
+    def list_share_replica_export_locations(self, share_replica,
+                                            columns=None, microversion=None):
+        """List share replica export locations.
+
+        :param share_replica: str -- ID of share replica.
+        :param columns: str -- comma separated string of columns.
+            Example, "--columns id,path".
+        :param microversion: API microversion to be used for request.
+        """
+        cmd = "share-replica-export-location-list %s" % share_replica
+        if columns is not None:
+            cmd += " --columns " + columns
+        export_locations_raw = self.manila(cmd, microversion=microversion)
+        export_locations = utils.listing(export_locations_raw)
+        return export_locations
+
+    @not_found_wrapper
+    @forbidden_wrapper
+    def get_share_replica_export_location(self, share_replica,
+                                          export_location_uuid,
+                                          microversion=None):
+        """Returns an export location by share replica and export location ID.
+
+        :param share_replica: str -- ID of share replica.
+        :param export_location_uuid: str -- UUID of an export location.
+        :param microversion: API microversion to be used for request.
+        """
+        export_raw = self.manila(
+            'share-replica-export-location-show '
+            '%(share_replica)s %(el_uuid)s' % {
+                'share_replica': share_replica,
+                'el_uuid': export_location_uuid,
+            },
+            microversion=microversion)
+        export = output_parser.details(export_raw)
+        return export
