@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import ast
 import six
 from tempest.lib.cli import output_parser
 import testtools
@@ -93,6 +94,15 @@ def listing(output_lines):
     return items
 
 
+def details(output_lines):
+    """Returns dict parsed from CLI output."""
+    result = listing(output_lines)
+    d = {}
+    for item in result:
+        d.update({item['Property']: item['Value']})
+    return d
+
+
 def is_microversion_supported(microversion):
     return (
         api_versions.APIVersion(CONF.min_api_microversion) <=
@@ -108,3 +118,38 @@ def skip_if_microversion_not_supported(microversion):
                   "allowed to be used by configuration." % microversion)
         return testtools.skip(reason)
     return lambda f: f
+
+
+def choose_matching_backend(share, pools, share_type):
+    extra_specs = {}
+
+    # convert extra-specs in provided type to dict format
+    pair = [x.strip() for x in share_type['required_extra_specs'].split(':')]
+    if len(pair) == 2:
+        value = (True if six.text_type(pair[1]).lower() == 'true'
+                 else False if six.text_type(pair[1]).lower() == 'false'
+                 else pair[1])
+        extra_specs[pair[0]] = value
+
+    selected_pool = next(
+        (x for x in pools if (x['Name'] != share['host'] and all(
+            y in ast.literal_eval(x['Capabilities']).items() for y in
+            extra_specs.items()))),
+        None)
+
+    return selected_pool['Name']
+
+
+def share_network_subnets_are_supported():
+    return is_microversion_supported('2.51')
+
+
+def get_subnet_by_availability_zone_name(client, share_network_id, az_name):
+    subnets = client.get_share_network_subnets(share_network_id)
+    return next((subnet for subnet in subnets
+                 if subnet['availability_zone'] == az_name), None)
+
+
+def get_default_subnet(client, share_network_id):
+    return get_subnet_by_availability_zone_name(client, share_network_id,
+                                                'None')
