@@ -13,8 +13,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import print_function
-
 import manilaclient
 from manilaclient import api_versions
 from manilaclient.tests.unit.v2 import fake_clients as fakes
@@ -38,6 +36,7 @@ class FakeClient(fakes.FakeClient):
         )
         self.client = FakeHTTPClient(**kwargs)
 
+
 fake_share_instance = {
     'id': 1234,
     'share_id': 'fake',
@@ -179,16 +178,17 @@ class FakeHTTPClient(fakes.FakeHTTPClient):
 
     def get_share_servers_1234(self, **kw):
         share_servers = {
-            'share_servers': {
+            'share_server': {
                 'id': 1234,
                 'share_network_id': 'fake_network_id_1',
+                'backend_details': {},
             },
         }
         return (200, {}, share_servers)
 
     def get_share_servers_5678(self, **kw):
         share_servers = {
-            'share_servers': {
+            'share_server': {
                 'id': 5678,
                 'share_network_id': 'fake_network_id_2',
             },
@@ -429,6 +429,63 @@ class FakeHTTPClient(fakes.FakeHTTPClient):
 
     post_shares_manage = post_os_share_manage
 
+    def post_share_servers_manage(self, body, **kw):
+        _body = {'share_server': {'id': 'fake'}}
+        resp = 202
+
+        if not ('host' in body['share_server']
+                and 'share_network' in body['share_server']
+                and 'identifier' in body['share_server']):
+            resp = 422
+
+        result = (resp, {}, _body)
+        return result
+
+    def post_share_servers_1234_action(self, body, **kw):
+        _body = None
+        assert len(list(body)) == 1
+        action = list(body)[0]
+
+        if action in ('reset_status', ):
+            assert 'status' in body.get(
+                'reset_status', body.get('os-reset_status'))
+            _body = {
+                'reset_status': {'status': body['reset_status']['status']}
+            }
+        elif action in ('unmanage', ):
+            assert 'force' in body[action]
+        elif action in (
+                'migration_cancel', 'migration_complete',
+                'migration_get_progress'):
+            assert body[action] is None
+            if 'migration_get_progress' == action:
+                _body = {'total_progress': 50,
+                         'task_state': 'fake_task_state',
+                         'destination_share_server_id': 'fake_dest_id'}
+                return 200, {}, _body
+            elif 'migration_complete' == action:
+                _body = {'destination_share_server_id': 'fake_dest_id'}
+                return 200, {}, _body
+        elif action in (
+                'migration_start', 'migration_check'):
+            assert 'host' in body[action]
+            if 'migration-check':
+                _body = {
+                    'compatible': True,
+                    'capacity': True,
+                    'capability': True,
+                    'writable': True,
+                    'nondisruptive': True,
+                    'preserve_snapshots': True,
+                }
+                return 200, {}, _body
+        elif action == 'reset_task_state':
+            assert 'task_state' in body[action]
+
+        resp = 202
+        result = (resp, {}, _body)
+        return result
+
     def post_os_share_unmanage_1234_unmanage(self, **kw):
         _body = None
         resp = 202
@@ -548,6 +605,9 @@ class FakeHTTPClient(fakes.FakeHTTPClient):
     def post_share_networks(self, **kwargs):
         return (202, {}, {'share_network': {}})
 
+    def post_share_networks_1234_subnets(self, **kwargs):
+        return (202, {}, {'share_network_subnet': {}})
+
     def post_shares(self, **kwargs):
         return (202, {}, {'share': {}})
 
@@ -578,6 +638,12 @@ class FakeHTTPClient(fakes.FakeHTTPClient):
     def delete_share_networks_fake_share_network2(self, **kwargs):
         return (202, {}, None)
 
+    def delete_share_networks_1234_subnets_fake_subnet1(self, **kwargs):
+        return (202, {}, None)
+
+    def delete_share_networks_1234_subnets_fake_subnet2(self, **kwargs):
+        return (202, {}, None)
+
     def delete_snapshots_fake_snapshot1(self, **kwargs):
         return (202, {}, None)
 
@@ -632,6 +698,14 @@ class FakeHTTPClient(fakes.FakeHTTPClient):
         }
         return (200, {}, share_nw)
 
+    def get_share_networks_1234_subnets_fake_subnet_id(self, **kw):
+        subnet = {
+            'share_network_subnet': {
+                'id': 'fake_subnet_id',
+            },
+        }
+        return (200, {}, subnet)
+
     def get_security_services(self, **kw):
         security_services = {
             'security_services': [
@@ -658,6 +732,7 @@ class FakeHTTPClient(fakes.FakeHTTPClient):
                     'domain': 'fake_domain',
                     'server': 'fake_server',
                     'dns_ip': 'fake_dns_ip',
+                    'ou': 'fake_ou',
                     'type': 'fake_type',
                     'status': 'fake_status',
                     'project_id': 'fake_project_id',
@@ -830,6 +905,26 @@ class FakeHTTPClient(fakes.FakeHTTPClient):
         replicas = {'share_replica': self.fake_share_replica}
         return (200, {}, replicas)
 
+    def get_share_replicas_5678_export_locations(self, **kw):
+        export_locations = {
+            'export_locations': [
+                get_fake_export_location(),
+            ]
+        }
+        return (200, {}, export_locations)
+
+    def get_share_replicas_1234_export_locations(self, **kw):
+        export_locations = {
+            'export_locations': [
+                get_fake_export_location(),
+            ]
+        }
+        return (200, {}, export_locations)
+
+    def get_share_replicas_1234_export_locations_fake_el_uuid(self, **kw):
+        export_location = {'export_location': get_fake_export_location()}
+        return (200, {}, export_location)
+
     def post_share_replicas(self, **kw):
         return (202, {}, {'share_replica': self.fake_share_replica})
 
@@ -873,8 +968,10 @@ class FakeHTTPClient(fakes.FakeHTTPClient):
 
     def get_types_1234(self, **kw):
         return (200, {}, {
-            'share_type': {'id': 1,
-                           'name': 'test-type-1',
+            'share_type': {'id': 1234,
+                           'name': 'test-type-1234',
+                           'share_type_access:is_public': True,
+                           'description': "test share type desc",
                            'extra_specs': {'test': 'test'},
                            'required_extra_specs': {'test': 'test'}}})
 
