@@ -15,8 +15,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from unittest import mock
+
 import ddt
-import mock
 
 from manilaclient import api_versions
 from manilaclient.common.apiclient import exceptions as client_exceptions
@@ -162,9 +163,11 @@ class SharesTest(utils.TestCase):
         ("2.7", "/shares/manage", None),
         ("2.8", "/shares/manage", True),
         ("2.8", "/shares/manage", False),
+        ("2.49", "/shares/manage", False, '1234'),
     )
     @ddt.unpack
-    def test_manage_share(self, microversion, resource_path, is_public=False):
+    def test_manage_share(self, microversion, resource_path, is_public=False,
+                          share_server_id=None):
         service_host = "fake_service_host"
         protocol = "fake_protocol"
         export_path = "fake_export_path"
@@ -180,6 +183,7 @@ class SharesTest(utils.TestCase):
             "driver_options": driver_options,
             "name": name,
             "description": description,
+            "share_server_id": share_server_id,
         }
         version = api_versions.APIVersion(microversion)
         if version >= api_versions.APIVersion('2.8'):
@@ -190,15 +194,19 @@ class SharesTest(utils.TestCase):
 
         with mock.patch.object(manager, "_create",
                                mock.Mock(return_value="fake")):
-
-            if version >= api_versions.APIVersion('2.8'):
+            if version < api_versions.APIVersion('2.8'):
+                result = manager.manage(
+                    service_host, protocol, export_path, driver_options,
+                    share_type, name, description)
+            elif (api_versions.APIVersion('2.8') <= version
+                    < api_versions.APIVersion('2.49')):
                 result = manager.manage(
                     service_host, protocol, export_path, driver_options,
                     share_type, name, description, is_public)
             else:
                 result = manager.manage(
                     service_host, protocol, export_path, driver_options,
-                    share_type, name, description)
+                    share_type, name, description, is_public, share_server_id)
 
             self.assertEqual(manager._create.return_value, result)
             manager._create.assert_called_once_with(
@@ -297,8 +305,8 @@ class SharesTest(utils.TestCase):
         cs.shares.list(detailed=False, search_opts=search_opts)
         cs.assert_called(
             'GET',
-            '/shares?description%7E=fake_description&fake_int=1&'
-            'fake_str=fake_str_value&is_public=True&name%7E=fake_name')
+            '/shares?description~=fake_description&fake_int=1&'
+            'fake_str=fake_str_value&is_public=True&name~=fake_name')
 
     @ddt.data(('id', 'b4991315-eb7d-43ec-979e-5715d4399827', True),
               ('id', 'b4991315-eb7d-43ec-979e-5715d4399827', False),
