@@ -13,15 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import six
-try:
-    from urllib import urlencode  # noqa
-except ImportError:
-    from urllib.parse import urlencode  # noqa
-
 from manilaclient import api_versions
 from manilaclient import base
-from manilaclient.openstack.common.apiclient import base as common_base
+from manilaclient.common.apiclient import base as common_base
 
 RESOURCE_PATH_LEGACY = '/os-services'
 RESOURCE_PATH = '/services'
@@ -47,12 +41,7 @@ class ServiceManager(base.Manager):
 
         :rtype: list of :class:`Service`
         """
-        query_string = ''
-        if search_opts:
-            query_string = urlencode(
-                sorted([(k, v) for (k, v) in six.iteritems(search_opts) if v]))
-            if query_string:
-                query_string = "?%s" % query_string
+        query_string = self._build_query_string(search_opts)
         return self._list(resource_path + query_string, RESOURCE_NAME)
 
     @api_versions.wraps("1.0", "2.6")
@@ -61,7 +50,7 @@ class ServiceManager(base.Manager):
             search_opts=search_opts, resource_path=RESOURCE_PATH_LEGACY)
 
     @api_versions.wraps("2.7")  # noqa
-    def list(self, search_opts=None):
+    def list(self, search_opts=None):  # noqa
         return self._do_list(
             search_opts=search_opts, resource_path=RESOURCE_PATH)
 
@@ -75,7 +64,7 @@ class ServiceManager(base.Manager):
         return self._do_enable(host, binary, RESOURCE_PATH_LEGACY)
 
     @api_versions.wraps("2.7")  # noqa
-    def enable(self, host, binary):
+    def enable(self, host, binary):  # noqa
         return self._do_enable(host, binary, RESOURCE_PATH)
 
     def _do_disable(self, host, binary, resource_path=RESOURCE_PATH):
@@ -88,7 +77,7 @@ class ServiceManager(base.Manager):
         return self._do_disable(host, binary, RESOURCE_PATH_LEGACY)
 
     @api_versions.wraps("2.7")  # noqa
-    def disable(self, host, binary):
+    def disable(self, host, binary):  # noqa
         return self._do_disable(host, binary, RESOURCE_PATH)
 
     def server_api_version(self, url_append=""):
