@@ -13,9 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo_utils import uuidutils
+
 from manilaclient import api_versions
 from manilaclient import base
-from manilaclient.openstack.common.apiclient import base as common_base
+from manilaclient.common.apiclient import base as common_base
 
 
 class ShareInstance(common_base.Resource):
@@ -46,10 +48,26 @@ class ShareInstanceManager(base.ManagerWithFind):
         share_id = common_base.getid(instance)
         return self._get("/share_instances/%s" % share_id, "share_instance")
 
-    @api_versions.wraps("2.3")
-    def list(self):
+    @api_versions.wraps("2.3", "2.34")
+    def list(self, search_opts=None):
         """List all share instances."""
-        return self._list('/share_instances', 'share_instances')
+        return self.do_list()
+
+    @api_versions.wraps("2.35")   # noqa
+    def list(self, export_location=None, search_opts=None):  # noqa
+        """List all share instances."""
+        return self.do_list(export_location)
+
+    def do_list(self, export_location=None):
+        """List all share instances."""
+        path = '/share_instances'
+        if export_location:
+            if uuidutils.is_uuid_like(export_location):
+                path += '?export_location_id=' + export_location
+            else:
+                path += '?export_location_path=' + export_location
+
+        return self._list(path, 'share_instances')
 
     def _action(self, action, instance, info=None, **kwargs):
         """Perform a share instance 'action'.
@@ -76,7 +94,7 @@ class ShareInstanceManager(base.ManagerWithFind):
         return self._do_force_delete(instance, "os-force_delete")
 
     @api_versions.wraps("2.7")  # noqa
-    def force_delete(self, instance):
+    def force_delete(self, instance):  # noqa
         return self._do_force_delete(instance, "force_delete")
 
     def _do_reset_state(self, instance, state, action_name):
@@ -92,5 +110,5 @@ class ShareInstanceManager(base.ManagerWithFind):
         return self._do_reset_state(instance, state, "os-reset_status")
 
     @api_versions.wraps("2.7")  # noqa
-    def reset_state(self, instance, state):
+    def reset_state(self, instance, state):  # noqa
         return self._do_reset_state(instance, state, "reset_status")
