@@ -17,13 +17,11 @@
 """
 Share Type interface.
 """
-from six.moves.urllib import parse
 
 from manilaclient import api_versions
 from manilaclient import base
 from manilaclient.common.apiclient import base as common_base
 from manilaclient import exceptions
-from manilaclient import utils
 
 
 class ShareType(common_base.Resource):
@@ -103,6 +101,10 @@ class ShareType(common_base.Resource):
             if resp is not None:
                 return resp
 
+    def update(self, **kwargs):
+        """Update this share type."""
+        return self.manager.update(self, **kwargs)
+
 
 class ShareTypeManager(base.ManagerWithFind):
     """Manage :class:`ShareType` resources."""
@@ -114,17 +116,10 @@ class ShareTypeManager(base.ManagerWithFind):
 
         :rtype: list of :class:`ShareType`.
         """
-        query_string = ''
+        search_opts = search_opts or {}
         if show_all:
-            query_string = '?is_public=all'
-        if search_opts:
-            search_opts = utils.unicode_key_value_to_string(search_opts)
-            params = sorted(
-                [(k, v) for (k, v) in list(search_opts.items()) if v])
-            if params and query_string:
-                query_string += "&%s" % parse.urlencode(params)
-            if params and not query_string:
-                query_string += "?%s" % parse.urlencode(params)
+            search_opts['is_public'] = 'all'
+        query_string = self._build_query_string(search_opts)
         return self._list("/types%s" % query_string, "share_types")
 
     def show(self, share_type):
@@ -173,6 +168,30 @@ class ShareTypeManager(base.ManagerWithFind):
             body["share_type"]["description"] = description
         return self._create("/types", body, "share_type")
 
+    def _do_update(self, share_type, name=None, is_public=None,
+                   is_public_keyname="share_type_access:is_public",
+                   description=None):
+        """Update the name and/or description for a share type.
+
+        :param share_type: the ID of the :class: `ShareType` to update.
+        :param name: Descriptive name of the share type.
+        :param description: Description of the share type.
+        :rtype: :class:`ShareType`
+        """
+
+        body = {
+            "share_type": {}
+        }
+
+        if name:
+            body["share_type"]["name"] = name
+        if is_public is not None:
+            body["share_type"][is_public_keyname] = is_public
+        if description or description == "":
+            body["share_type"]["description"] = description
+        return self._update("/types/%s" % common_base.getid(share_type),
+                            body, "share_type")
+
     @api_versions.wraps("1.0", "2.6")
     def create(self, name, spec_driver_handles_share_servers,
                spec_snapshot_support=None, is_public=True, extra_specs=None):
@@ -190,7 +209,7 @@ class ShareTypeManager(base.ManagerWithFind):
             is_public_keyname="os-share-type-access:is_public")
 
     @api_versions.wraps("2.7", "2.23")  # noqa
-    def create(self, name, spec_driver_handles_share_servers,
+    def create(self, name, spec_driver_handles_share_servers,  # noqa
                spec_snapshot_support=None, is_public=True, extra_specs=None):
 
         if extra_specs is None:
@@ -204,7 +223,7 @@ class ShareTypeManager(base.ManagerWithFind):
         return self._do_create(name, extra_specs, is_public)
 
     @api_versions.wraps("2.24", "2.40")  # noqa
-    def create(self, name, spec_driver_handles_share_servers,
+    def create(self, name, spec_driver_handles_share_servers,  # noqa
                spec_snapshot_support=None, is_public=True, extra_specs=None):
 
         if extra_specs is None:
@@ -217,7 +236,7 @@ class ShareTypeManager(base.ManagerWithFind):
         return self._do_create(name, extra_specs, is_public)
 
     @api_versions.wraps("2.41")  # noqa
-    def create(self, name, spec_driver_handles_share_servers,
+    def create(self, name, spec_driver_handles_share_servers,  # noqa
                spec_snapshot_support=None, is_public=True, extra_specs=None,
                description=None):
         if extra_specs is None:
@@ -230,6 +249,11 @@ class ShareTypeManager(base.ManagerWithFind):
         return self._do_create(name, extra_specs, is_public,
                                description=description)
 
+    @api_versions.wraps("2.50")
+    def update(self, share_type, name=None, is_public=None, description=None):
+        return self._do_update(share_type, name, is_public,
+                               description=description)
+
     def _handle_spec_driver_handles_share_servers(
             self, extra_specs, spec_driver_handles_share_servers):
         """Validation and default for DHSS extra spec."""
