@@ -19,7 +19,6 @@ import ipaddress
 from oslo_utils import uuidutils
 import re
 import six
-from six.moves.urllib import parse
 import string
 
 from manilaclient import api_versions
@@ -27,7 +26,6 @@ from manilaclient import base
 from manilaclient.common.apiclient import base as common_base
 from manilaclient.common import constants
 from manilaclient import exceptions
-from manilaclient import utils
 from manilaclient.v2 import share_instances
 
 
@@ -217,7 +215,7 @@ class ShareManager(base.ManagerWithFind):
     def _do_manage(self, service_host, protocol, export_path,
                    driver_options=None, share_type=None,
                    name=None, description=None, is_public=None,
-                   resource_path="/shares/manage"):
+                   share_server_id=None, resource_path="/shares/manage"):
         """Manage some existing share.
 
         :param service_host: text - host where manila share service is running
@@ -228,6 +226,7 @@ class ShareManager(base.ManagerWithFind):
         :param name: text - name of new share
         :param description: - description for new share
         :param is_public: - visibility for new share
+        :param share_server_id: text - id of share server associated with share
         """
         driver_options = driver_options if driver_options else dict()
         body = {
@@ -238,6 +237,7 @@ class ShareManager(base.ManagerWithFind):
             'driver_options': driver_options,
             'name': name,
             'description': description,
+            'share_server_id': share_server_id,
         }
 
         if is_public is not None:
@@ -248,25 +248,36 @@ class ShareManager(base.ManagerWithFind):
     @api_versions.wraps("1.0", "2.6")
     def manage(self, service_host, protocol, export_path, driver_options=None,
                share_type=None, name=None, description=None):
-        is_public = None
         return self._do_manage(
-            service_host, protocol, export_path, driver_options, share_type,
-            name, description, is_public, resource_path="/os-share-manage")
+            service_host, protocol, export_path, driver_options=driver_options,
+            share_type=share_type, name=name, description=description,
+            resource_path="/os-share-manage")
 
     @api_versions.wraps("2.7", "2.7")  # noqa
-    def manage(self, service_host, protocol, export_path, driver_options=None,
+    def manage(self, service_host, protocol, export_path, driver_options=None,  # noqa
                share_type=None, name=None, description=None):
-        is_public = None
         return self._do_manage(
-            service_host, protocol, export_path, driver_options, share_type,
-            name, description, is_public, resource_path="/shares/manage")
+            service_host, protocol, export_path, driver_options=driver_options,
+            share_type=share_type, name=name, description=description,
+            resource_path="/shares/manage")
 
-    @api_versions.wraps("2.8")  # noqa
-    def manage(self, service_host, protocol, export_path, driver_options=None,
+    @api_versions.wraps("2.8", "2.48")  # noqa
+    def manage(self, service_host, protocol, export_path, driver_options=None,  # noqa
                share_type=None, name=None, description=None, is_public=False):
         return self._do_manage(
-            service_host, protocol, export_path, driver_options, share_type,
-            name, description, is_public, "/shares/manage")
+            service_host, protocol, export_path, driver_options=driver_options,
+            share_type=share_type, name=name, description=description,
+            is_public=is_public, resource_path="/shares/manage")
+
+    @api_versions.wraps("2.49")  # noqa
+    def manage(self, service_host, protocol, export_path, driver_options=None,  # noqa
+               share_type=None, name=None, description=None, is_public=False,
+               share_server_id=None):
+        return self._do_manage(
+            service_host, protocol, export_path, driver_options=driver_options,
+            share_type=share_type, name=name, description=description,
+            is_public=is_public, share_server_id=share_server_id,
+            resource_path="/shares/manage")
 
     @api_versions.wraps("1.0", "2.6")
     def unmanage(self, share):
@@ -278,7 +289,7 @@ class ShareManager(base.ManagerWithFind):
             "/os-share-unmanage/%s/unmanage" % common_base.getid(share))
 
     @api_versions.wraps("2.7")  # noqa
-    def unmanage(self, share):
+    def unmanage(self, share):   # noqa
         """Unmanage a share.
 
         :param share: either share object or text with its ID.
@@ -330,7 +341,7 @@ class ShareManager(base.ManagerWithFind):
                             sort_key=sort_key, sort_dir=sort_dir)
 
     @api_versions.wraps("2.35")   # noqa
-    def list(self, detailed=True, search_opts=None,
+    def list(self, detailed=True, search_opts=None,   # noqa
              sort_key=None, sort_dir=None):
         """Get a list of all shares."""
         return self.do_list(detailed=detailed, search_opts=search_opts,
@@ -398,13 +409,7 @@ class ShareManager(base.ManagerWithFind):
             else:
                 search_opts['export_location_path'] = export_location
 
-        query_string = ""
-        if search_opts:
-            search_opts = utils.unicode_key_value_to_string(search_opts)
-            params = sorted(
-                [(k, v) for (k, v) in list(search_opts.items()) if v])
-            if params:
-                query_string = "?%s" % parse.urlencode(params)
+        query_string = self._build_query_string(search_opts)
 
         if detailed:
             path = "/shares/detail%s" % (query_string,)
@@ -437,7 +442,7 @@ class ShareManager(base.ManagerWithFind):
         return self._do_force_delete(share, "os-force_delete")
 
     @api_versions.wraps("2.7")  # noqa
-    def force_delete(self, share):
+    def force_delete(self, share):   # noqa
         return self._do_force_delete(share, "force_delete")
 
     @staticmethod
@@ -456,8 +461,8 @@ class ShareManager(base.ManagerWithFind):
     '''
     @staticmethod
     def _validate_username(access):
-        sole_periods_spaces_re = '[\s|\.]+$'
-        valid_username_re = '.[^\"\/\\\[\]\:\;\|\=\,\+\*\?\<\>]{3,254}$'
+        sole_periods_spaces_re = r'[\s|\.]+$'
+        valid_username_re = r'.[^\"\/\\\[\]\:\;\|\=\,\+\*\?\<\>]{3,254}$'
         username = access
 
         if re.match(sole_periods_spaces_re, username):
@@ -468,7 +473,7 @@ class ShareManager(base.ManagerWithFind):
         if not re.match(valid_username_re, username):
             exc_str = ('Invalid user or group name. Must be 4-255 characters '
                        'and consist of alphanumeric characters and '
-                       'exclude special characters "/\[]:;|=,+*?<>')
+                       'exclude special characters "/\\[]:;|=,+*?<>')
             raise exceptions.CommandError(exc_str)
 
     @staticmethod
@@ -550,20 +555,20 @@ class ShareManager(base.ManagerWithFind):
             share, access_type, access, access_level, "os-allow_access")
 
     @api_versions.wraps("2.7", "2.12")  # noqa
-    def allow(self, share, access_type, access, access_level, metadata=None):
+    def allow(self, share, access_type, access, access_level, metadata=None):   # noqa
         self._validate_access(access_type, access)
         return self._do_allow(
             share, access_type, access, access_level, "allow_access")
 
     @api_versions.wraps("2.13", "2.37")  # noqa
-    def allow(self, share, access_type, access, access_level, metadata=None):
+    def allow(self, share, access_type, access, access_level, metadata=None):   # noqa
         valid_access_types = ('ip', 'user', 'cert', 'cephx')
         self._validate_access(access_type, access, valid_access_types)
         return self._do_allow(
             share, access_type, access, access_level, "allow_access")
 
     @api_versions.wraps("2.38", "2.44")  # noqa
-    def allow(self, share, access_type, access, access_level, metadata=None):
+    def allow(self, share, access_type, access, access_level, metadata=None):   # noqa
         valid_access_types = ('ip', 'user', 'cert', 'cephx')
         self._validate_access(access_type, access, valid_access_types,
                               enable_ipv6=True)
@@ -571,7 +576,7 @@ class ShareManager(base.ManagerWithFind):
             share, access_type, access, access_level, "allow_access")
 
     @api_versions.wraps("2.45")  # noqa
-    def allow(self, share, access_type, access, access_level, metadata=None):
+    def allow(self, share, access_type, access, access_level, metadata=None):   # noqa
         valid_access_types = ('ip', 'user', 'cert', 'cephx')
         self._validate_access(access_type, access, valid_access_types,
                               enable_ipv6=True)
@@ -591,7 +596,7 @@ class ShareManager(base.ManagerWithFind):
         return self._do_deny(share, access_id, "os-deny_access")
 
     @api_versions.wraps("2.7")  # noqa
-    def deny(self, share, access_id):
+    def deny(self, share, access_id):   # noqa
         return self._do_deny(share, access_id, "deny_access")
 
     def _do_access_list(self, share, action_name):
@@ -611,7 +616,7 @@ class ShareManager(base.ManagerWithFind):
         return self._do_access_list(share, "os-access_list")
 
     @api_versions.wraps("2.7", "2.44")  # noqa
-    def access_list(self, share):
+    def access_list(self, share):   # noqa
         return self._do_access_list(share, "access_list")
 
     def get_metadata(self, share):
@@ -679,7 +684,7 @@ class ShareManager(base.ManagerWithFind):
         return self._do_reset_state(share, state, "os-reset_status")
 
     @api_versions.wraps("2.7")  # noqa
-    def reset_state(self, share, state):
+    def reset_state(self, share, state):  # noqa
         return self._do_reset_state(share, state, "reset_status")
 
     def _do_extend(self, share, new_size, action_name):
@@ -695,7 +700,7 @@ class ShareManager(base.ManagerWithFind):
         return self._do_extend(share, new_size, "os-extend")
 
     @api_versions.wraps("2.7")  # noqa
-    def extend(self, share, new_size):
+    def extend(self, share, new_size):  # noqa
         return self._do_extend(share, new_size, "extend")
 
     def _do_shrink(self, share, new_size, action_name):
@@ -711,7 +716,7 @@ class ShareManager(base.ManagerWithFind):
         return self._do_shrink(share, new_size, "os-shrink")
 
     @api_versions.wraps("2.7")  # noqa
-    def shrink(self, share, new_size):
+    def shrink(self, share, new_size):   # noqa
         return self._do_shrink(share, new_size, "shrink")
 
     def list_instances(self, share):
